# -*- coding: UTF-8 -*-
import numpy as np
import matplotlib.pyplot as plt
from scipy.spatial.distance import cdist
import time
from mpl_toolkits.mplot3d import Axes3D
def isCorePoint(ptIndex,floorIds,xyArr,timeArr,eps1,eps2,minPts):
    currentFloorId = floorIds[ptIndex]
    xyDisArr = compute_xy_dis(ptIndex,xyArr)
    timeDisArr = compute_time_dis(ptIndex,timeArr)
    flag = (xyDisArr <= eps1) & (timeDisArr <= eps2) & (floorIds == currentFloorId)
    if np.sum(np.where(flag, 1, 0)) >= minPts:
        return True
    return False
def compute_xy_dis(ptIndex,xyArr):
    xyDisArr = cdist(xyArr[ptIndex].reshape(-1,2), xyArr, metric='euclidean')
    return xyDisArr[0]
def compute_time_dis(ptIndex,timeArr):
    timeDisArr = cdist(timeArr[ptIndex].reshape(-1, 1), timeArr.reshape(-1, 1), metric='euclidean')
    return timeDisArr[0]
def updateSeeds(seeds,neighbours,labels,start):
    for neighbour in neighbours:
        if labels[neighbour] == -1 and neighbour >= start:
            seeds.add(neighbour)
    return seeds

def Indoor_STDBSCAN(data,eps1,eps2,minPts):
    n, m = data.shape
    timeArr=data[:,0]
    xyArr = data[:,1:3]
    floorIds = data[:,3]
    labels = np.full((n,), -1)
    clusterId = 0
    start = 0
    for ptIndex in range(n):
        if ptIndex < start:
            continue
        if labels[ptIndex] != -1:
            continue
        if not isCorePoint(ptIndex, floorIds, xyArr, timeArr, eps1, eps2, minPts):
            continue
        labels[ptIndex] = clusterId

        currentFloorId = floorIds[ptIndex]
        neighbours = np.where((compute_xy_dis(ptIndex,xyArr) <= eps1) & (compute_time_dis(ptIndex,timeArr) <= eps2) & (labels == -1) & ((floorIds == currentFloorId)))[0]
        seeds = set()
        seeds = updateSeeds(seeds, neighbours, labels, start)
        while len(seeds) > 0:
            newPoint = seeds.pop()
            labels[newPoint] = clusterId
            queryResults = np.where((compute_xy_dis(newPoint,xyArr) <= eps1) & (compute_time_dis(newPoint,timeArr) <= eps2) & (floorIds == floorIds[newPoint]))[0]
            if len(queryResults) >= minPts:
                seeds = updateSeeds(seeds, queryResults, labels, start)
        clusterId = clusterId + 1
        start = np.argwhere(labels != -1)[-1][0]
    return labels
def plotFeature(data, labels_):
    clusterNum=len(set(labels_))
    ax = plt.figure().add_subplot(111, projection='3d')
    scatterColors = ['red', 'blue', 'orange', 'orchid', 'black', '#E0E3E6', '#E0E0E6','#E5E0E6','#E6E0E4','#E6E0E4','brown']
    for i in range(-1,clusterNum):
        colorSytle = scatterColors[i % len(scatterColors)]
        subCluster = data[np.where(labels_==i)]
        ax.scatter(subCluster[:,0], subCluster[:,1],subCluster[:,2], c=colorSytle, s=20)
    plt.show()